<?php
/**
 * Class for BackWPup cron methods
 */
class BackWPup_Cron {

	/**
	 * @static
	 *
	 * @param $arg
	 * @internal param $args
	 */
	public static function run( $arg ) {

		$job_object = BackWPup_Job::get_working_data();
		if ( is_object( $job_object ) ) {
			//reschedule restart
			wp_schedule_single_event( time() + 60, 'backwpup_cron', array( 'id' => 'restart' ) );
			//restart job if not working
			$not_worked_time = microtime( TRUE ) - $job_object->timestamp_last_update;
			if ( $not_worked_time > 300 )
				BackWPup_Job::start_wp_cron( 0 );
		}
		elseif ( $arg != 'restart' ) {
			//check that job exits
			$jobids = BackWPup_Option::get_job_ids( 'activetype', 'wpcron' );
			if ( ! in_array( $arg, $jobids) )
				return;
			//reschedule job for next run
			$cronnxet = BackWPup_Cron::cron_next( BackWPup_Option::get( $arg, 'cron' ) );
			wp_schedule_single_event( $cronnxet, 'backwpup_cron', array( 'id' => $arg ) );
			//start job
			BackWPup_Job::start_wp_cron( $arg );
		}
	}


	/**
	 * Check Jobs worked and Cleanup logs and so on
	 */
	public static function check_cleanup() {

		$job_object = BackWPup_Job::get_working_data( TRUE );
		$jobids = BackWPup_Option::get_job_ids( );

		// check aborted jobs for longer than a tow hours, abort them courtly and send mail
		if ( is_object( $job_object ) ) {
			$not_worked_time = microtime( TRUE ) - $job_object->timestamp_last_update;
			if ( $not_worked_time > 7200 ) {
				unlink( BackWPup::get_plugin_data( 'running_file' ) );
				//add log entry
				$timestamp = "<span title=\"[Type: " . E_USER_ERROR . "|Line: " . __LINE__ . "|File: " . basename( __FILE__ ) . "|PID: " . $job_object->pid . "]\">[" . date_i18n( 'd-M-Y H:i:s' ) . "]</span> ";
				file_put_contents( $job_object->logfile, $timestamp . "<span class=\"error\">" . __( 'ERROR:', 'backwpup' ) . " " . __( 'Aborted, because 2 hours no progress!', 'backwpup' ) . "</span><br />" . PHP_EOL, FILE_APPEND );
				//write new log header
				$job_object->errors ++;
				$fd      = fopen( $job_object->logfile, 'r+' );
				$filepos = ftell( $fd );
				while ( ! feof( $fd ) ) {
					$line = fgets( $fd );
					if ( stripos( $line, "<meta name=\"backwpup_errors\"" ) !== FALSE ) {
						fseek( $fd, $filepos );
						fwrite( $fd, str_pad( "<meta name=\"backwpup_errors\" content=\"" . $job_object->errors . "\" />", 100 ) . PHP_EOL );
						break;
					}
					$filepos = ftell( $fd );
				}
				fclose( $fd );
				//update job settings
				if ( ! empty( $job_object->job[ 'jobid' ] ) )
					BackWPup_Option::update( $job_object->job[ 'jobid' ], 'lastruntime', ( current_time( 'timestamp' ) - $job_object->start_time ) );
				//clean up temp
				if ( ! empty( $job_object->backup_file ) && is_file( BackWPup::get_plugin_data( 'TEMP' ) . $job_object->backup_file ) )
					unlink( BackWPup::get_plugin_data( 'TEMP' ) . $job_object->backup_file );
				if ( ! empty( $job_object->folder_list_file ) && is_file( $job_object->folder_list_file ) )
					unlink( $job_object->folder_list_file );
				if ( ! empty( $job_object->additional_files_to_backup ) ) {
					foreach ( $job_object->additional_files_to_backup as $additional_file ) {
						if ( $additional_file == BackWPup::get_plugin_data( 'TEMP' ) . basename( $additional_file ) && is_file( $additional_file ) )
							unlink( $additional_file );
					}
				}
				//Send mail with log
				if ( $job_object->job[ 'mailaddresslog' ] ) {
					$subject = sprintf( __( '[%3$s] BackWPup log %1$s: %2$s', 'backwpup' ), date_i18n( 'd-M-Y H:i', $job_object->job[ 'lastrun' ], TRUE ), esc_attr( $job_object->job[ 'name' ] ), __('ERROR','backwpup') );
					$headers = array();
					$headers[] = 'Content-Type: text/html; charset='. get_bloginfo( 'charset' );
					$headers[] = 'X-Priority: 1';
					if ( ! empty( $job_object->job[ 'mailaddresssenderlog' ] ) )
						$headers[] = 'From: ' . $job_object->job[ 'mailaddresssenderlog' ];
					wp_mail( $job_object->job[ 'mailaddresslog' ], $subject, file_get_contents( $job_object->logfile ), $headers );
				}
			}
		}

		//Compress not compressed logs
		if ( function_exists( 'gzopen' ) && ! $job_object && BackWPup_Option::get(  'cfg', 'gzlogs' )) {
			//Compress logs from last Jobs
			foreach ( $jobids as $jobid ) {
				$log_file = BackWPup_Option::get( $jobid, 'logfile' );
				//compress uncompressed
				if ( is_file( $log_file ) && '.html' == substr( $log_file, -5 ) ) {
					$compress = new BackWPup_Create_Archive( $log_file . '.gz' );
					if ( $compress->add_file( $log_file ) ) {
						BackWPup_Option::update( $jobid, 'logfile', $log_file. '.gz' );
						unlink( $log_file );
					}
					unset( $compress );
				}

			}
			//Compress old not compressed logs
			$log_folder = scandir( BackWPup_Option::get( 'cfg', 'logfolder' ) );
			foreach ( $log_folder as $log_file ) {
				if ( is_file( BackWPup_Option::get( 'cfg', 'logfolder' ) . $log_file ) && '.html' == substr( $log_file, -5 ) ) {
					$compress = new BackWPup_Create_Archive(  BackWPup_Option::get( 'cfg', 'logfolder' ) . $log_file . '.gz' );
					if ( $compress->add_file( BackWPup_Option::get( 'cfg', 'logfolder' ) . $log_file ) ) {
						unlink( BackWPup_Option::get( 'cfg', 'logfolder' ) . $log_file );
					}
					unset( $compress );
				}
			}
		}

		//Jobs cleanings
		if ( ! $job_object ) {
			//remove restart cron
			wp_clear_scheduled_hook( 'backwpup_cron', array( 'id' => 'restart' ) );
			//clear maintenance mode
			if ( is_file( ABSPATH . '.maintenance' ) or ( defined( 'FB_WM_TEXTDOMAIN' ) && ( get_site_option( FB_WM_TEXTDOMAIN . '-msqld' ) == 1 or get_option( FB_WM_TEXTDOMAIN . '-msqld' ) == 1 ) ) ) {
				if ( class_exists( 'WPMaintenanceMode' ) ) { //Support for WP Maintenance Mode Plugin (Frank Bueltge)
					if ( is_multisite() && is_plugin_active_for_network( FB_WM_BASENAME ) )
						update_site_option( FB_WM_TEXTDOMAIN . '-msqld', 0 );
					else
						update_option( FB_WM_TEXTDOMAIN . '-msqld', 0 );
				}
				else { //WP Support
					unlink( ABSPATH . '.maintenance' );
				}
			}
			//temp cleanup
			$temp_folder = scandir( BackWPup::get_plugin_data( 'TEMP' ) );
			foreach ( $temp_folder as $temp_file ) {
				if ( is_file( BackWPup::get_plugin_data( 'TEMP' ) . $temp_file ) ) {
					if ( '.gz' == substr( $temp_file, -3 ) || '.sql' == substr( $temp_file, -4 ) || '.bz2' == substr( $temp_file, -4 ) || '.zip' == substr( $temp_file, -4 ) || '.txt' == substr( $temp_file, -4 ) || '.xml' == substr( $temp_file, -4 ) )
						unlink( BackWPup::get_plugin_data( 'TEMP' ) . $temp_file );
				}
			}
		}
	}

	/**
	 *
	 * Get the local time timestamp of the next cron execution
	 *
	 * @param string $cronstring string of cron (* * * * *)
	 *
	 * @return int timestamp
	 */
	public static function cron_next( $cronstring ) {

		$cron      = array();
		$cronarray = array();
		//Cron string
		list( $cronstr[ 'minutes' ], $cronstr[ 'hours' ], $cronstr[ 'mday' ], $cronstr[ 'mon' ], $cronstr[ 'wday' ] ) = explode( ' ', $cronstring, 5 );

		//make arrays form string
		foreach ( $cronstr as $key => $value ) {
			if ( strstr( $value, ',' ) )
				$cronarray[ $key ] = explode( ',', $value );
			else
				$cronarray[ $key ] = array( 0 => $value );
		}

		//make arrays complete with ranges and steps
		foreach ( $cronarray as $cronarraykey => $cronarrayvalue ) {
			$cron[ $cronarraykey ] = array();
			foreach ( $cronarrayvalue as $value ) {
				//steps
				$step = 1;
				if ( strstr( $value, '/' ) )
					list( $value, $step ) = explode( '/', $value, 2 );
				//replace weekday 7 with 0 for sundays
				if ( $cronarraykey == 'wday' )
					$value = str_replace( '7', '0', $value );
				//ranges
				if ( strstr( $value, '-' ) ) {
					list( $first, $last ) = explode( '-', $value, 2 );
					if ( ! is_numeric( $first ) || ! is_numeric( $last ) || $last > 60 || $first > 60 ) //check
						return 2147483647;
					if ( $cronarraykey == 'minutes' && $step < 5 ) //set step minimum to 5 min.
						$step = 5;
					$range = array();
					for ( $i = $first; $i <= $last; $i = $i + $step ) {
						$range[ ] = $i;
					}
					$cron[ $cronarraykey ] = array_merge( $cron[ $cronarraykey ], $range );
				}
				elseif ( $value == '*' ) {
					$range = array();
					if ( $cronarraykey == 'minutes' ) {
						if ( $step < 10 ) //set step minimum to 5 min.
							$step = 10;
						for ( $i = 0; $i <= 59; $i = $i + $step ) {
							$range[ ] = $i;
						}
					}
					if ( $cronarraykey == 'hours' ) {
						for ( $i = 0; $i <= 23; $i = $i + $step ) {
							$range[ ] = $i;
						}
					}
					if ( $cronarraykey == 'mday' ) {
						for ( $i = $step; $i <= 31; $i = $i + $step ) {
							$range[ ] = $i;
						}
					}
					if ( $cronarraykey == 'mon' ) {
						for ( $i = $step; $i <= 12; $i = $i + $step ) {
							$range[ ] = $i;
						}
					}
					if ( $cronarraykey == 'wday' ) {
						for ( $i = 0; $i <= 6; $i = $i + $step ) {
							$range[ ] = $i;
						}
					}
					$cron[ $cronarraykey ] = array_merge( $cron[ $cronarraykey ], $range );
				}
				else {
					//Month names
					if ( strtolower( $value ) == 'jan' )
						$value = 1;
					if ( strtolower( $value ) == 'feb' )
						$value = 2;
					if ( strtolower( $value ) == 'mar' )
						$value = 3;
					if ( strtolower( $value ) == 'apr' )
						$value = 4;
					if ( strtolower( $value ) == 'may' )
						$value = 5;
					if ( strtolower( $value ) == 'jun' )
						$value = 6;
					if ( strtolower( $value ) == 'jul' )
						$value = 7;
					if ( strtolower( $value ) == 'aug' )
						$value = 8;
					if ( strtolower( $value ) == 'sep' )
						$value = 9;
					if ( strtolower( $value ) == 'oct' )
						$value = 10;
					if ( strtolower( $value ) == 'nov' )
						$value = 11;
					if ( strtolower( $value ) == 'dec' )
						$value = 12;
					//Week Day names
					if ( strtolower( $value ) == 'sun' )
						$value = 0;
					if ( strtolower( $value ) == 'sat' )
						$value = 6;
					if ( strtolower( $value ) == 'mon' )
						$value = 1;
					if ( strtolower( $value ) == 'tue' )
						$value = 2;
					if ( strtolower( $value ) == 'wed' )
						$value = 3;
					if ( strtolower( $value ) == 'thu' )
						$value = 4;
					if ( strtolower( $value ) == 'fri' )
						$value = 5;
					if ( ! is_numeric( $value ) || $value > 60 ) //check
						return 2147483647;
					$cron[ $cronarraykey ] = array_merge( $cron[ $cronarraykey ], array( 0 => $value ) );
				}
			}
		}

		//generate  years
		for ( $i = gmdate( 'Y' ); $i < gmdate( 'Y', 2147483647 ); $i ++ ) {
			$cron[ 'year' ][ ] = $i;
		}

		//calc next timestamp
		$current_timestamp = time();
		foreach ( $cron[ 'year' ] as $year ) {
			foreach ( $cron[ 'mon' ] as $mon ) {
				foreach ( $cron[ 'mday' ] as $mday ) {
					foreach ( $cron[ 'hours' ] as $hours ) {
						foreach ( $cron[ 'minutes' ] as $minutes ) {
							$timestamp = gmmktime( $hours - get_option( 'gmt_offset' ), $minutes, 0, $mon, $mday, $year );
							if ( $timestamp && in_array( gmdate( 'j', $timestamp ), $cron[ 'mday' ] ) && in_array( gmdate( 'w', $timestamp ), $cron[ 'wday' ] ) && $timestamp > $current_timestamp ) {
								return $timestamp;
							}
						}
					}
				}
			}
		}

		return 2147483647;
	}

}
